CREATE DATABASE IF NOT EXISTS `qstats`  DEFAULT CHARACTER SET 'UTF8';
USE `qstats`;

SET NAMES 'UTF8';

CREATE TABLE IF NOT EXISTS `acl` (
  `id` int(11) NOT NULL auto_increment,
  `resource` varchar(50) default NULL,
  `level` varchar(20) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `byresource` (`resource`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `designer` (
  `id` int(11) NOT NULL auto_increment,
  `keyword` varchar(50) default NULL,
  `parameter` varchar(250) default NULL,
  `value` text,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `keypar` (`keyword`,`parameter`),
  KEY `keyindex` (`keyword`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `language` (
  `id` int(6) NOT NULL auto_increment,
  `iso_code` varchar(5) collate utf8_bin NOT NULL default 'en',
  `string` text collate utf8_bin,
  `translation` text collate utf8_bin,
  `pending` tinyint(1) default '1',
  `timestamp` timestamp NOT NULL default CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `idx` (`iso_code`,`string`(64)),
  KEY `stindex` (`string`(200))
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

CREATE TABLE IF NOT EXISTS `qagent` (
  `agent_id` int(6) NOT NULL auto_increment,
  `agent` varchar(40) NOT NULL default '',
  PRIMARY KEY  (`agent_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `qevent` (
  `event_id` int(11) NOT NULL auto_increment,
  `event` varchar(40) default NULL,
  PRIMARY KEY  (`event_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `qname` (
  `queue_id` int(11) NOT NULL auto_increment,
  `queue` varchar(40) NOT NULL default '',
  PRIMARY KEY  (`queue_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `queue_stats` (
  `queue_stats_id` int(12) NOT NULL auto_increment,
  `uniqueid` varchar(40) default NULL,
  `datetime` timestamp NOT NULL,
  `qname` int(6) default NULL,
  `qagent` int(6) default NULL,
  `qevent` int(2) default NULL,
  `info1` varchar(50) NOT NULL,
  `info2` varchar(50) NOT NULL,
  `info3` varchar(50) NOT NULL,
  `info4` varchar(50) NOT NULL,
  `info5` varchar(50) NOT NULL,
  PRIMARY KEY  (`queue_stats_id`),
  UNIQUE KEY `unico` (`uniqueid`,`datetime`,`qname`,`qagent`,`qevent`),
  KEY `ixevent` (`qevent`),
  KEY `ixdate` (`datetime`),
  KEY `ixuni` (`uniqueid`),
  KEY `ixagent` (`qagent`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `recordings` (
  `uniqueid` varchar(40) NOT NULL default '',
  `filename` varchar(200) default NULL,
  PRIMARY KEY  (`uniqueid`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `sched` (
  `schid` int(6) NOT NULL auto_increment,
  `dest` text,
  `reports` text,
  `lastdays` smallint(4) default NULL,
  `field` varchar(40) default '',
  `exceed` varchar(40) default '',
  `crondaymonth` varchar(40) default '*',
  `crondayweek` varchar(40) default '*',
  `cronhour` varchar(20) default '*',
  `cronminute` varchar(20) default '*',
  `lastrun` datetime,
  `active` tinyint(4) default '1',
  `queues` TEXT default NULL, 
  PRIMARY KEY  (`schid`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `secure_level` (
  `level` int(11) NOT NULL default '0',
  `detail` varchar(100) default NULL,
  PRIMARY KEY  (`level`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `setup` (
  `id` int(11) NOT NULL auto_increment,
  `keyword` varchar(50) default NULL,
  `parameter` varchar(50) default NULL,
  `value` varchar(255) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `keypar` (`keyword`,`parameter`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `sqlrealtime` (
  `lastupdate` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `data` longtext,
  `user` varchar(100) NOT NULL,
  PRIMARY KEY  (`user`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `user_filter` (
  `user` varchar(30) default NULL,
  `param` varchar(30) default NULL,
  `value` varchar(100) default NULL
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `userqagent` (
  `users_id` int(6) default NULL,
  `qagent_agent_id` int(6) default NULL
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `userqname` (
  `users_id` int(6) default NULL,
  `qname_queue_id` int(6) default NULL
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `users` (
  `id` int(11) NOT NULL auto_increment,
  `login` varchar(30) default NULL,
  `password` varchar(100) default NULL,
  `name` varchar(100) default NULL,
  `level` varchar(20) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `nombre` (`login`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `gqname` (
  `groupname_id` int(11) NOT NULL auto_increment,
  `groupname` varchar(200) default NULL,
  PRIMARY KEY  (`groupname_id`),
  UNIQUE KEY `name` (`groupname`)
);

CREATE TABLE IF NOT EXISTS `gqnameqname` (
  `gqname_groupname_id` int(11) default NULL,
  `qname_queue_id` int(11) default NULL
);

CREATE TABLE IF NOT EXISTS `plugins` (
  `id` int(11) NOT NULL auto_increment,
  `rawname` varchar(100) NOT NULL,
  `name` varchar(100) NOT NULL,
  `description` tinytext NOT NULL,
  `enabled` tinyint(1) default '1',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `name` (`name`)
);

CREATE TABLE IF NOT EXISTS `agent_activity` (
  `id` int(11) NOT NULL auto_increment,
  `datetime` datetime default NULL,
  `queue` varchar(80) default NULL,
  `agent` varchar(100) default NULL,
  `event` varchar(40) default NULL,
  `data` varchar(150) default NULL,
  `lastedforseconds` int(11) default NULL,
  `uniqueid` varchar(100) default NULL,
  `computed` tinyint(1) default NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `uni` (`datetime`,`queue`,`agent`,`event`,`uniqueid`)
);

CREATE OR REPLACE view queue_stats_full AS select q.*,qevent.event AS event, qagent.agent AS agent, qname.queue AS queue from queue_stats q FORCE INDEX (ixdate) LEFT JOIN qagent ON qagent=agent_id LEFT JOIN qname ON qname=queue_id LEFT JOIN qevent ON qevent = event_id;

DROP TRIGGER IF EXISTS queue_stats_ins;
DELIMITER $$
CREATE TRIGGER queue_stats_ins
AFTER INSERT ON queue_stats
FOR EACH ROW
BEGIN

SET @event_name = ''; SET @queue_name = ''; SET @agent_name = ''; SET @enterdate  = '';
SET @connectdate= ''; SET @clid       = ''; SET @position   = ''; SET @url        = '';
SET @overflow   = ''; SET @firstenter = ''; SET @lastenter  = ''; SET @did        = '';

SELECT event FROM qevent WHERE event_id = NEW.qevent INTO @event_name;
SELECT agent FROM qagent WHERE agent_id = NEW.qagent INTO @agent_name;
SELECT queue FROM qname  WHERE queue_id = NEW.qname  INTO @queue_name;

IF @event_name LIKE 'COMPLETE%' THEN

/*answered calls*/

/* Elige el primer evento ENTERQUEUE, y si elijo el ultimo? (DESC) */
SELECT datetime,info1,info2,info3,info5 FROM queue_stats JOIN qevent ON qevent=event_id WHERE event='ENTERQUEUE' 
AND uniqueid=NEW.uniqueid ORDER BY datetime LIMIT 1 INTO @enterdate,@url,@clid,@position,@did;

/* Elige el ultimo evento CONNECT */
SELECT datetime FROM queue_stats JOIN qevent ON qevent=event_id 
WHERE event='CONNECT' AND uniqueid=NEW.uniqueid ORDER BY datetime DESC LIMIT 1 INTO @connectdate;

SELECT IF(count(uniqueid)>1,1,0) FROM queue_stats JOIN qevent ON qevent=event_id WHERE event='ENTERQUEUE'
AND uniqueid=NEW.uniqueid INTO @overflow;

/*
SELECT COUNT(*),SUBSTRING_INDEX(GROUP_CONCAT(datetime),',',1),SUBSTRING_INDEX(GROUP_CONCAT(datetime),',',-1) 
FROM queue_stats LEFT JOIN qevent ON qevent = qevent.event_id  WHERE uniqueid=NEW.uniqueid AND event='ENTERQUEUE' 
INTO @overflow,@firstenter,@lastenter;
*/

REPLACE INTO queue_stats_mv (uniqueid,event,agent,queue,datetime,datetimeconnect,datetimeend,clid,position,url,did,overflow,info1,info2,info3,info4,info5) 
VALUES (NEW.uniqueid,@event_name,@agent_name,@queue_name,@enterdate,@connectdate,NEW.datetime,@clid,IFNULL(NEW.info3,1),IFNULL(@url,''),IFNULL(@did,''),
@overflow,IFNULL(NEW.info1,''),IFNULL(NEW.info2,''),IFNULL(NEW.info3,''),IFNULL(NEW.info4,''),IFNULL(NEW.info5,''));

ELSEIF @event_name='TRANSFER' THEN

/*transferred calls*/

SELECT datetime,info1,info2,info3,info4,info5 FROM queue_stats JOIN qevent ON qevent=event_id WHERE event='ENTERQUEUE' 
AND uniqueid=NEW.uniqueid ORDER BY datetime LIMIT 1 INTO @enterdate,@url,@clid,@position,@info4,@did;

SELECT datetime FROM queue_stats JOIN qevent ON qevent=event_id 
WHERE event='CONNECT' AND uniqueid=NEW.uniqueid ORDER BY datetime DESC LIMIT 1 INTO @connectdate;

SELECT count(*) FROM queue_stats JOIN qevent ON qevent=event_id WHERE event='ENTERQUEUE'
AND uniqueid=NEW.uniqueid INTO @overflow;

REPLACE INTO queue_stats_mv (uniqueid,event,agent,queue,datetime,datetimeconnect,datetimeend,clid,position,url,did,info1,info2,overflow,info3,info4,info5) 
VALUES (NEW.uniqueid,@event_name,@agent_name,@queue_name,@enterdate,@connectdate,NEW.datetime,@clid,IFNULL(@position,1),IFNULL(@url,''),IFNULL(@did,''),
IFNULL(NEW.info1,0),IFNULL(NEW.info2,0),@overflow,IFNULL(NEW.info3,''),IFNULL(NEW.info4,''),IFNULL(NEW.info5,''));

ELSEIF @event_name LIKE '%ABANDON%' OR @event_name LIKE 'EXIT%' THEN

/*unanswered calls*/

SELECT datetime,info1,info2,info3,info5 FROM queue_stats JOIN qevent ON qevent=event_id WHERE event='ENTERQUEUE' 
AND uniqueid=NEW.uniqueid ORDER BY datetime LIMIT 1 INTO @enterdate,@url,@clid,@position,@did;

SELECT count(*) FROM queue_stats JOIN qevent ON qevent=event_id WHERE event='ENTERQUEUE'
AND uniqueid=NEW.uniqueid INTO @overflow;

SET @connectdate = @enterdate;

REPLACE INTO queue_stats_mv (uniqueid,event,agent,queue,datetime,datetimeconnect,datetimeend,clid,position,url,did,info1,info2,overflow,info3,info4,info5) 
VALUES (NEW.uniqueid,@event_name,@agent_name,@queue_name,@enterdate,@connectdate,NEW.datetime,IFNULL(@clid,''),IFNULL(@position,1),IFNULL(@url,''),IFNULL(@did,''),
IFNULL(NEW.info1,0),IFNULL(NEW.info2,0),@overflow,IFNULL(NEW.info3,''),IFNULL(NEW.info4,''),IFNULL(NEW.info5,''));


ELSEIF @event_name LIKE 'AGENT%' OR @event_name LIKE '%PAUSE%' OR @event_name LIKE '%MEMBER%' THEN 

/* login, pausa, etc */

INSERT INTO queue_stats_mv (uniqueid,event,agent,queue,datetime,datetimeconnect,datetimeend,info1,info2,info3,info4,info5) 
VALUES (CONCAT_WS('.','x',UNIX_TIMESTAMP(NOW()),NEW.queue_stats_id),@event_name,@agent_name,@queue_name,NEW.datetime,NEW.datetime,NEW.datetime,
IFNULL(NEW.info1,0),IFNULL(NEW.info2,0),IFNULL(NEW.info3,''),IFNULL(NEW.info4,''),IFNULL(NEW.info5,''));

END IF;
END;
$$
DELIMITER ;

CREATE TABLE `qstats.queue_stats_mv` (
  `id` int(11) unsigned NOT NULL auto_increment,
  `datetime` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
  `datetimeconnect` timestamp NOT NULL default '0000-00-00 00:00:00',
  `datetimeend` timestamp NOT NULL default '0000-00-00 00:00:00',
  `queue` varchar(100) NOT NULL default '',
  `agent` varchar(100) NOT NULL default '',
  `event` varchar(40) NOT NULL default '',
  `uniqueid` varchar(50) NOT NULL default '',
  `clid` varchar(50) NOT NULL default '',
  `url` varchar(100) NOT NULL default '',
  `did` varchar(100) NOT NULL default '',
  `position` int(6) unsigned NOT NULL default '1',
  `info1` varchar(50) NOT NULL default '',
  `info2` varchar(50) NOT NULL default '',
  `info3` varchar(50) NOT NULL default '',
  `info4` varchar(50) NOT NULL default '',
  `info5` varchar(50) NOT NULL default '',
  `overflow` int(6) unsigned NOT NULL default '1',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `uni` (`uniqueid`),
  KEY `fecha` (`datetime`),
  KEY `ev` (`event`)
);

